/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#ifndef MIRAI_MIRAI_H
#define MIRAI_MIRAI_H

#include "BaseFileResource.h"
#include "TaskItem.h"
#include "TodoMd.h"
#include "mirai-core/BaseResource.h"
#include <algorithm>
#include <functional>
#include <map>
#include <memory>
#include <optional>
#include <string>

namespace mirai
{

class Mirai
{

  public:
	void loadResource(std::unique_ptr<BaseResource> &&resource);
	void unloadAllResources();
	void save();
	void removeTask(const TaskItem *taskItem);
	std::optional<std::reference_wrapper<BaseResource>> getResourceByName(const std::string &name);

	std::vector<std::unique_ptr<BaseResource>> &getResources();
	const std::vector<std::string> &getTags();

	void reloadTags();

	// new stuff

	// Returns a non owning pointer to the requested resource or nullptr if not found.
	BaseResource *getResourceById(int id)
	{
		if (id >= resources_.size()) {
			return nullptr;
		}
		return resources_.at(id).get();
	}

  private:
	std::vector<std::unique_ptr<BaseResource>> resources_;
	std::vector<std::string> tags_;
};
} // namespace mirai
#endif
